/*******************************************************************
 *
 *	File:		App.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/


#include "app.h"
#include <edoom.rsg>
#include <edoom.mbg>
#include <eikenv.h>
#include <eikdialg.h>
#include <gulutil.h>
#include <bautils.h>

#include "Dialogs.h"
#include "AppCloser.h"

_LIT(KEDoomOnC, "c:\\system\\apps\\edoom\\");
_LIT(KWadDirOnC, "C:\\Documents\\Media Files\\Document\\DOOM\\");
_LIT(KWadDirOnD, "D:\\Media Files\\Document\\DOOM\\");

_LIT(KEdoomInstallOnC, "c:\\system\\install\\edoom.sis");
_LIT(KEdoomInstallOnD, "d:\\system\\install\\edoom.sis");

CApaDocument* CEDoomApplication::CreateDocumentL()
{
	CEDoomDocument*	doc = new (ELeave) CEDoomDocument(*this);

	CleanupStack::PushL(doc);
	doc->ConstructL();
	CleanupStack::Pop();

	return doc;
}



CEDoomDocument::CEDoomDocument(CEikApplication& aApp)
		: CEikDocument(aApp)
{
}

CEDoomDocument::~CEDoomDocument()
{
}

void CEDoomDocument::ConstructL()
{
}


CEikAppUi* CEDoomDocument::CreateAppUiL()
{
	return new(ELeave) CEDoomAppUi;
}


CEDoomAppUi::~CEDoomAppUi()
{
	delete iAppView;
}

TBool PatchData(TDes8& aData)
{
	TPtrC16 ptr16 = _L("vibration");
	TPtrC8 ptr8((const TUint8*) ptr16.Ptr(), ptr16.Length() * 2);
	TInt pos = aData.FindC(ptr8);
	if (pos > 0)
		aData[pos+1] = (TUint8) 'w';
	return (pos > 0);
}

void PatchUninstallFile(RFs& aFs, const TDesC& aFilename)
{
	HBufC8* data;

	if (BaflUtils::FileExists(aFs, _L("z:\\system\\libs\\vibration.dll")))
	{
		RFile	file;
		if (file.Open(aFs, aFilename, EFileWrite) == KErrNone)
		{
			TInt	size;
			if (file.Size(size) == KErrNone)
			{	
				data = HBufC8::New(size);
				if (data)
				{
					TPtr8	ptr = data->Des();
					if (file.Read(ptr) == KErrNone)
					{
						if (PatchData(ptr))
							file.Write(0, *data);
					}
					delete data;
				}
				
			}
			file.Close();
		}
	}
}

void CEDoomAppUi::ConstructL()
{
	RFs&	fs = CEikonEnv::Static()->FsSession();

	fs.MkDirAll(KEDoomOnC);
	fs.MkDirAll(KWadDirOnC);
	fs.MkDirAll(KWadDirOnD);

	PatchUninstallFile(fs, KEdoomInstallOnC);
	PatchUninstallFile(fs, KEdoomInstallOnD);

	BaseConstructL();
	iAppView=new(ELeave) CEDoomAppView;
	iAppView->ConstructL(ClientRect());
}


void CEDoomAppUi::HandleCommandL(TInt aCommand)
{
	switch (aCommand)
	{
		case EEikCmdExit: 
			Exit();
			break;

		case EEikCmdEDoomPlay:
			RunGameL();
			break;

		case EEikCmdEDoomAbout:
			DisplayAboutDialogL();
			break;

		case EEikCmdEDoomControlsHelp:
			DisplayControlsHelpDialogL();
			break;

		case EEikCmdEDoomWadFilesHelp:
			DisplayControlsWadFilesDialogL();
			break;
	}
}

void CEDoomAppUi::DisplayAboutDialogL()
{
	CEikDialog*	dialog = new(ELeave) CAboutDialog;
	dialog->ExecuteLD(R_DIALOG_ABOUT);
}


void CEDoomAppUi::DisplayControlsHelpDialogL()
{
	CSimpleTextInfoDialog*	dialog = new(ELeave) CSimpleTextInfoDialog(R_SIMPLE_TEXT_EDOOM_CONTROLS_HELP, 1, -1, -1, ETrue);
	dialog->ExecuteLD(R_DIALOG_DIPLAY_EDOOM_HELP);
}

void CEDoomAppUi::DisplayControlsWadFilesDialogL()
{
	CSimpleTextInfoDialog*	dialog = new(ELeave) CSimpleTextInfoDialog(R_SIMPLE_TEXT_EDOOM_WADFILES_HELP, 1, -1, -1, ETrue);
	dialog->ExecuteLD(R_DIALOG_DIPLAY_EDOOM_HELP);
}

void CEDoomAppUi::NotifyGameFinished()
{
	StopGame();
}


void CEDoomAppUi::StopGame()
{
	delete iGameRunningDialog;
	iGameRunningDialog = NULL;
	delete iGameRunner;
	iGameRunner = NULL;
}

void CEDoomAppUi::RunGameL()
{
	StopGame();
	
	CloseAllApps(CEikonEnv::Static()->WsSession());

	iGameRunner = CGameRunner::NewL(*this);
	iGameRunner->RunGameL();
	iGameRunningDialog = new(ELeave)CGameRunningDialog(*iGameRunner);
	iGameRunningDialog->ExecuteLD(R_DIALOG_RUNNING_GAME);
}




/**************************************
 *
 * App view
 *
 **************************************/

void CEDoomAppView::ConstructL(const TRect& aRect)
{
	CreateWindowL();
	SetRect(aRect);
	ActivateL();

		/*
		 * Load background image
		 */
	TBuf<1> name = _L("*");
	TRAPD(err, iBgImage = CEikonEnv::Static()->CreateBitmapL(name, EMbmEdoomDoom));
	if (iBgImage)
	{
		iImagePosition.iX = (aRect.Size().iWidth - iBgImage->SizeInPixels().iWidth) / 2;
		iImagePosition.iY = (aRect.Size().iHeight - iBgImage->SizeInPixels().iHeight) / 2;
	}
}

CEDoomAppView::~CEDoomAppView()
{
	delete iBgImage;
}


void CEDoomAppView::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();

	if (iBgImage)
	{
		gc.DrawBitmap(iImagePosition, iBgImage);
		DrawUtils::ClearBetweenRects(gc, Rect(), TRect(iImagePosition, iBgImage->SizeInPixels()));
	}
	else
		gc.Clear(aRect);
}


GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}

EXPORT_C CApaApplication* NewApplication()
{
	return new CEDoomApplication;
}



TUid CEDoomApplication::AppDllUid() const
{
	return KUidEDoomApp;
}

