/*******************************************************************
 *
 *	File:		Audio.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "audio.h"
#include "polledas.h"

#include <f32file.h>
#include <e32svr.h>
#include <string.h>

#include <stdio.h>




/*******************************************
 *
 * CGameAudio
 *
 *******************************************/

CGameAudio::CGameAudio(TBool aStereo, TInt aPcmFrames) : iStereo(aStereo), iPcmFrames(aPcmFrames)
{
}


CGameAudio* CGameAudio::NewL(TBool aStereo, TInt aPcmFrames)
{
	CGameAudio*		self = new(ELeave) CGameAudio(aStereo, aPcmFrames);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();		// self
	return self;
}


CGameAudio::~CGameAudio()
{
	Destruct();
}

void CGameAudio::Destruct()
{
	delete iMdaAudioOutputStream;
	iMdaAudioOutputStream = NULL;

	for (TInt i=0 ; i<KSoundBuffers ; i++)
		delete iSoundBuffers[i];
}


void CGameAudio::ConstructL()
{
	CMdaServer* server = CMdaServer::NewL();

	iMdaAudioOutputStream = CMdaAudioOutputStream::NewL(*this, server);
	iMdaAudioOutputStream->Open(&iMdaAudioDataSettings);//callback calls QueueIdle function

	WaitForOpenToCompleteL();

	iMdaAudioOutputStream->SetAudioPropertiesL(TMdaAudioDataSettings::ESampleRate11025Hz, iStereo ? TMdaAudioDataSettings::EChannelsStereo : TMdaAudioDataSettings::EChannelsMono);

	TInt	bytesPerFrame = iStereo ? iPcmFrames << 2 : iPcmFrames << 1;
	for (TInt i=0 ; i<KSoundBuffers ; i++)
	{
		iSoundBuffers[i] = HBufC8::NewL(bytesPerFrame * KBufferedFrames);
		iSoundBuffers[i]->Des().SetLength(bytesPerFrame * KBufferedFrames);
	}

	iFrameCount = 0;
	iCurrentBuffer = 0;
	iCurrentPosition = (TInt16*) iSoundBuffers[iCurrentBuffer]->Ptr();
}

TInt CGameAudio::Write(TInt16* aBuffer, TInt aSize)
{
	TInt	byteSize = iStereo ? aSize << 2 : aSize << 1;
	Mem::Copy(iCurrentPosition, aBuffer, byteSize);
	iCurrentPosition += aSize;

	if (++iFrameCount == KBufferedFrames)
	{
		iHasCopied = EFalse;
		iMdaAudioOutputStream->WriteL(*iSoundBuffers[iCurrentBuffer]);
		User::After(0);

		iFrameCount = 0;
		if (++iCurrentBuffer == KSoundBuffers)
			iCurrentBuffer = 0;
		iCurrentPosition = (TInt16*) iSoundBuffers[iCurrentBuffer]->Ptr();
	}
	return aSize;
}

void CGameAudio::WaitForCopyToCompleteL()
{
	while (!iHasCopied)
		CPolledActiveScheduler::Instance()->Schedule();
}

void CGameAudio::WaitForOpenToCompleteL()
{
	TInt	count = 20;		// 5 seconds
	TInt	waitPeriod = 250 * 1000;
	while (!iIsOpen && --count)
	{
		User::After(waitPeriod);
		CPolledActiveScheduler::Instance()->Schedule();
	}
	if (!iIsOpen)
		User::LeaveIfError(KErrNotSupported);
}

void CGameAudio::MaoscOpenComplete(TInt aError)
{
//	if (aError)
//		RDebug::Print(_L("CGameAudio::MaoscOpenComplete, error=%d"), aError);
	iIsOpen = ETrue;
}

void CGameAudio::MaoscBufferCopied(TInt aError, const TDesC8& /*aBuffer*/)
{
	iHasCopied = ETrue;
//	if (aError)
//		RDebug::Print(_L("CGameAudio::MaoscBufferCopied, error=%d"), aError);
}

void CGameAudio::MaoscPlayComplete(TInt aError)
{
//	if (aError)
//		RDebug::Print(_L("CGameAudio::MaoscPlayComplete, error=%d"), aError);
}
