/*******************************************************************
 *
 *	File:		I_Video.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <stdlib.h>
#include <sys\unistd.h>
#include "video.h"


extern "C"
{
#include "doomdef.h"
#include "i_system.h"
#include "m_argv.h"
#include "v_video.h"
#include "st_stuff.h"
#include "i_video.h"
#include "keys.h"
#include "g_input.h"

// profiler support
#include "ll_video.h"
#include "profiler.h"

extern void D_PostEvent (const event_t* ev);


/*********************************************************
 *
 * Global variables we are supposed to declare
 *
 *********************************************************/

// this is the CURRENT rendermode!! very important : used by w_wad, and many other code
rendermode_t    rendermode = render_soft;
boolean         highcolor = false;
byte			graphics_started = 0;

void I_UpdateNoBlit (void){}	// not needed
void I_StartFrame (void){}		// not needed

void I_ShutdownGraphics(void);
void I_FinishUpdate(void);
int VID_SetMode (int modenum);
void I_StartupGraphics(void);
void I_SetPalette (byte* palette);
void I_ReadScreen (byte* scr);
char *VID_GetModeName (int modenum);
int VID_NumModes(void);
int VID_GetModeForSize( int w, int h);


int IsPenDown();		// needed in g_game.c 
}		// extern "C"




/*********************************************************
 *
 * Global variables (for both video & keyboard)
 *
 *********************************************************/

static TInt sProfiler = 0;
void ToggleProfiler()
{
	if (sProfiler == 0)
	{
		profiler_start();
		sProfiler = 1;
	} 
	else
	{
		profiler_stop();
		sProfiler = 0;
	}
}


/*********************************************************
 *
 * Toolbar
 *
 *********************************************************/

#ifdef __WINS__
_LIT(KDoomMbmFile_C, "z:\\system\\Programs\\doom.mbm");
_LIT(KDoomMbmFile_D, "c:\\system\\Programs\\doom.mbm");
#else
_LIT(KDoomMbmFile_C, "c:\\system\\apps\\edoom\\doom.mbm");
_LIT(KDoomMbmFile_D, "d:\\system\\apps\\edoom\\doom.mbm");
#endif

const TInt KMaxVirtualButtons	= 15;
const TInt KButtonMargin		= 4;
const TInt KButtonFixedHeight	= 8;

const TInt KVirtualKeyboardHotkey = 240;

class CToolbar : public CBase, public MScreenOrientationObserver, public MGameScreenDrawObserver
{
public:
	static CToolbar* NewL(RWsSession& aWsSession, CGameScreenDevice* aGameScreenDevice);
	static TVirtualButton	iVirtualButtons[KMaxVirtualButtons+1];

public:			// implements MScreenOrientationObserver
	virtual void ScreenOrientationChanged(TOrientation aOrientation);

public:			// implements MGameScreenDrawObserver
	virtual TBool	WantDrawToScreen() {return EFalse;}
	virtual void	PreDrawFrameToScreen(CGameScreenDevice* /*aDstGameScreenDevice*/){};
	virtual void	PostDrawFrameToScreen(CGameScreenDevice* aDstGameScreenDevice);

protected:
	CToolbar(RWsSession& aWsSession, CGameScreenDevice* aGameScreenDevice);
	~CToolbar();
	void ConstructL();
	void DrawToolBarL();
	void DrawButtonL(TInt aButton);

	CFbsGameScreenDevice*	iBitmap;
	RWsSession&				iWsSession;
	CGameScreenDevice*		iLastGameScreenDevice;
	CGameScreenDevice*		iGameScreenDevice;
	TBool					iIsRotLeft;

	TSize					iSize;
	TPoint					iPoint;

	TInt					iButtonWidth[KMaxVirtualButtons];
};

TVirtualButton	CToolbar::iVirtualButtons[KMaxVirtualButtons+1] =
{
	{0, 0, 0, 0, EStdKeyEnter,	EKeyEnter},
	{0, 0, 0, 0, EStdKeyEscape, EKeyEscape},
	{0, 0, 0, 0, KVirtualKeyboardHotkey, KVirtualKeyboardHotkey},
	{0, 0, 0, 0, 1,				'1'},
	{0, 0, 0, 0, 2,				'2'},
	{0, 0, 0, 0, 3,				'3'},
	{0, 0, 0, 0, 4,				'4'},
	{0, 0, 0, 0, 5,				'5'},
	{0, 0, 0, 0, 6,				'6'},
	{0, 0, 0, 0, 7,				'7'},
	{0, 0, 0, 0, EStdKeyTab,	EKeyTab},
	{0, 0, 0, 0, 'y',			'y'},
	{0, 0, 0, 0, 'n',			'n'},
	{0, 0, 0, 0, EStdKeyF2,		KEY_F2},
	{0, 0, 0, 0, EStdKeyF3,		KEY_F3},
	{-1, -1, -1, -1, -1, -1}
};

CToolbar::CToolbar(RWsSession& aWsSession, CGameScreenDevice* aGameScreenDevice) 
	: iWsSession(aWsSession), iGameScreenDevice(aGameScreenDevice)
{
}

CToolbar::~CToolbar()
{
	delete iBitmap;
}

CToolbar* CToolbar::NewL(RWsSession& aWsSession, CGameScreenDevice* aGameScreenDevice)
{
	CToolbar*	self = new(ELeave)CToolbar(aWsSession, aGameScreenDevice);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}


void CToolbar::ConstructL()
{
	iBitmap = CFbsGameScreenDevice::NewL(TSize(320, 8), iGameScreenDevice->DisplayMode());
	iBitmap->Clear();
	
	iSize.iHeight = KButtonFixedHeight;
	DrawToolBarL();
}

void CToolbar::ScreenOrientationChanged(TOrientation aOrientation)
{
	iIsRotLeft = (aOrientation == EOrientationRotLeft);
	iLastGameScreenDevice = NULL;	// forces  a redraw of the toolbar

	if (iIsRotLeft)
	{
		iPoint = TPoint(207, 0);

		iVirtualButtons[0].iTlX = 200;
		iVirtualButtons[0].iTlY = 0;
		iVirtualButtons[0].iBrX = 208;
		iVirtualButtons[0].iBrY = iButtonWidth[0];
		for (TInt i=1 ; i<KMaxVirtualButtons ; i++)
		{
			iVirtualButtons[i].iTlX = 200;
			iVirtualButtons[i].iTlY = iVirtualButtons[i-1].iBrY + KButtonMargin;
			iVirtualButtons[i].iBrX = 208;
			iVirtualButtons[i].iBrY = iVirtualButtons[i].iTlY + iButtonWidth[i];
		}
	}
	else
	{
		iVirtualButtons[0].iBrX = 8;
		iVirtualButtons[0].iBrY = 320;
		iVirtualButtons[0].iTlX = 0;
		iVirtualButtons[0].iTlY = 320  - iButtonWidth[0];
		for (TInt i=1 ; i<KMaxVirtualButtons ; i++)
		{
			iVirtualButtons[i].iBrX = 8;
			iVirtualButtons[i].iBrY = iVirtualButtons[i-1].iTlY - KButtonMargin;
			iVirtualButtons[i].iTlX = 0;
			iVirtualButtons[i].iTlY = iVirtualButtons[i].iBrY - iButtonWidth[i];
		}

		iPoint = TPoint(0, 319);
	}
}

void CToolbar::DrawButtonL(TInt aButton)
{
	CWsBitmap*	bitmap = new(ELeave)CWsBitmap(iWsSession);
	CleanupStack::PushL(bitmap);

	TInt err = bitmap->Load(KDoomMbmFile_C, aButton);
	if (err)
		err = bitmap->Load(KDoomMbmFile_D, aButton);
	User::LeaveIfError(err);
	TSize size = bitmap->SizeInPixels();
	if (size.iHeight != 8)
		User::Leave(KErrTooBig);

	CFbsBitGc*	gc = iBitmap->FbsBitGc();
	gc->BitBlt(TPoint(iSize.iWidth + KButtonMargin, 0), bitmap);
	iSize.iWidth += KButtonMargin + size.iWidth;
	iButtonWidth[aButton] = size.iWidth;

	CleanupStack::PopAndDestroy(bitmap);
}

void CToolbar::DrawToolBarL()
{
	for (TInt i=0 ; i<KMaxVirtualButtons ; i++)
		DrawButtonL(i);
}

void CToolbar::PostDrawFrameToScreen(CGameScreenDevice* aDstGameScreenDevice)
{
	if ((iLastGameScreenDevice != aDstGameScreenDevice) || (aDstGameScreenDevice!=iGameScreenDevice))
	{
		if (iIsRotLeft)
			aDstGameScreenDevice->DirectDraw_16bpp_16bppRotLeft(iPoint, *iBitmap, TPoint(0, 0), iSize);
		else
			aDstGameScreenDevice->DirectDraw_16bpp_16bppRotRight(iPoint, *iBitmap, TPoint(0, 0), iSize);
		iLastGameScreenDevice = aDstGameScreenDevice;
	}

};


/*********************************************************
 *
 * Framedrawer
 *
 *********************************************************/

class TDoomFrameDrawer : public MGameScreenDrawer, public MScreenOrientationObserver
{
public:		// implements MGameScreenDrawer
	virtual void	DrawFrame(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice);
	virtual TRect	GameVisibleArea() const;
	virtual TSize	GameSize() const;
	virtual TPoint	GameOrigin() const;
	virtual void	SetGameOrigin(const TPoint& aOrigin);

public:		// implements MScreenOrientationObserver
	virtual void ScreenOrientationChanged(TOrientation aOrientation);

	void BlitImageRotLeft(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice);
	void BlitImageRotRight(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice);

	TBool		iIsRotLeft;
};

void TDoomFrameDrawer::DrawFrame(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
	if (sProfiler)
		profiler_show();

	PROFILER_MARK(PROFILER_7);
	if (iIsRotLeft)
		BlitImageRotLeft(aDstGameScreenDevice, aSrcGameScreenDevice);
	else
		BlitImageRotRight(aDstGameScreenDevice, aSrcGameScreenDevice);
	PROFILER_MARK(PROFILER_END);
}

TRect TDoomFrameDrawer::GameVisibleArea() const
{
	return TRect(GameOrigin(), GameSize());
}

TSize TDoomFrameDrawer::GameSize() const
{
#ifdef __WINS__
	return TSize(208, 320);
#else
	return TSize(200, 320);
#endif
}

TPoint	TDoomFrameDrawer::GameOrigin() const
{
#ifdef __WINS__
	return TPoint(0,0);
#else
	return iIsRotLeft ? TPoint(0,0) : TPoint(8,0);
#endif
}

void TDoomFrameDrawer::SetGameOrigin(const TPoint& /*aOrigin*/)
{
}

void TDoomFrameDrawer::BlitImageRotLeft(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
	TInt					width = aSrcGameScreenDevice->Size().iWidth;
	TInt					height = aSrcGameScreenDevice->Size().iHeight;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(height-1, 0));
	register TUint8*		src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	for(int i=0 ; i<height ; i++)
	{
		TUint16*	dst = frameBufferLine;
		for(int j=0 ; j<width ; j++)
		{
			*dst = (TUint16) pal[*src++];
			dst += aDstGameScreenDevice->BytesPerLine()>>1;
		}
		frameBufferLine--;
	}
}

void TDoomFrameDrawer::BlitImageRotRight(CGameScreenDevice* aDstGameScreenDevice, const CGameScreenDevice* aSrcGameScreenDevice)
{
	TInt					width = aSrcGameScreenDevice->Size().iWidth;
	TInt					height = aSrcGameScreenDevice->Size().iHeight;

	register TUint16*		frameBufferLine = (TUint16*) TGameBitmapUtil::PixelAddress(aDstGameScreenDevice, TPoint(8, width-1));
	register TUint8*		src = (TUint8*) aSrcGameScreenDevice->ScanLineAddress(0);
	TUint*					pal = aSrcGameScreenDevice->Palette()->PaletteBuffer();

	for(int i=0 ; i<height ; i++)
	{
		TUint16*	dst = frameBufferLine;
		for(int j=0 ; j<width ; j++)
		{
			*dst = (TUint16) pal[*src++];
			dst -= aDstGameScreenDevice->BytesPerLine()>>1;
		}
		frameBufferLine++;
	}
}


void TDoomFrameDrawer::ScreenOrientationChanged(TOrientation aOrientation)
{
	iIsRotLeft = aOrientation == EOrientationRotLeft;
}



/****************************************************
 *
 * IMPLEMENTATION OF TDoomControlInputHandler
 *
 ****************************************************/

//const TInt KMaxLastDeltas = 3;
const TInt KDeferJogdialUpEventsDeltaFrames = 3;

class TDoomControlInputHandler : public MGameControlInputHandler, public MScreenOrientationObserver, public MGameScreenDrawObserver
{
public:
	TDoomControlInputHandler();
	TInt MapToKeyCode(TInt aScancode);
	void PostKeyEvent(TInt aKeyCode, TBool aUpEvent);

public:			// implements MScreenOrientationObserver
	virtual void ScreenOrientationChanged(TOrientation aOrientation);

public:		// implements MGameControlInputHandler
	virtual void HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent);
	virtual void HandlePointerEvent(const TPointerEvent& /*aPointerEvent*/){};

public:		// implements MGameScreenDrawObserver
	virtual TBool	WantDrawToScreen() {return EFalse;}
	virtual void	PreDrawFrameToScreen(CGameScreenDevice* aDstGameScreenDevice);
	virtual void	PostDrawFrameToScreen(CGameScreenDevice* /*aDstGameScreenDevice*/){};

protected:
	static const TInt sKeyMapTableRotLeft[][2];
	static const TInt sKeyMapTableRotRight[][2];
	
	TBool		iIsRotLeft;
	TInt		iJogdialDownInFrame[2];
	TInt		iFrames;

	TBool		isMovingForward;
};


const TInt TDoomControlInputHandler::sKeyMapTableRotLeft[][2] = 
{
	{GAME_LEFT_SCANCODE, KEY_LEFTARROW},
	{GAME_RIGHT_SCANCODE, KEY_RIGHTARROW},
	{GAME_UP_SCANCODE, KEY_UPARROW},
	{GAME_DOWN_SCANCODE, KEY_DOWNARROW},

	{EStdKeyEscape, KEY_ESCAPE},
	{EStdKeyEnter, KEY_ENTER},

	{DEVICE_UP_SCANCODE, 'A'},
	{DEVICE_DOWN_SCANCODE, 'B'},
	{DEVICE_LEFT_SCANCODE, 'C'},
	{DEVICE_RIGHT_SCANCODE, 'D'},
	{DEVICE_SELECT_SCANCODE, 'E'},

	{EStdKeyApplication0, 'F'},
	{EStdKeyApplication1, 'G'},

	{EStdKeyTab, KEY_TAB},

	{0, 0}
};

const TInt TDoomControlInputHandler::sKeyMapTableRotRight[][2] = 
{
	{GAME_LEFT_SCANCODE, KEY_LEFTARROW},
	{GAME_RIGHT_SCANCODE, KEY_RIGHTARROW},
	{GAME_UP_SCANCODE, KEY_UPARROW},
	{GAME_DOWN_SCANCODE, KEY_DOWNARROW},

	{EStdKeyEscape, KEY_ESCAPE},
	{EStdKeyEnter, KEY_ENTER},

	{DEVICE_DOWN_SCANCODE, 'A'},
	{DEVICE_UP_SCANCODE, 'B'},
	{DEVICE_RIGHT_SCANCODE, 'C'},
	{DEVICE_LEFT_SCANCODE, 'D'},
	{DEVICE_SELECT_KEYCODE, 'E'},

	{EStdKeyApplication0, 'F'},
	{EStdKeyApplication1, 'G'},

	{EStdKeyTab, KEY_TAB},

	{0, 0}
};

TDoomControlInputHandler::TDoomControlInputHandler()
{
	iJogdialDownInFrame[0] = -1 - KDeferJogdialUpEventsDeltaFrames;
	iJogdialDownInFrame[1] = -1 - KDeferJogdialUpEventsDeltaFrames;
}

void TDoomControlInputHandler::ScreenOrientationChanged(TOrientation aOrientation)
{
	iIsRotLeft = (aOrientation == EOrientationRotLeft);
}

TInt TDoomControlInputHandler::MapToKeyCode(TInt aScancode)
{
	int i;

	if (iIsRotLeft)
	{
		for (i = 0; sKeyMapTableRotLeft[i][0]; i++)
			if (sKeyMapTableRotLeft[i][0] == aScancode)
				return sKeyMapTableRotLeft[i][1];
	}
	else
	{
		for (i = 0; sKeyMapTableRotRight[i][0]; i++)
			if (sKeyMapTableRotRight[i][0] == aScancode)
				return sKeyMapTableRotRight[i][1];
	}
	return aScancode;
}


void TDoomControlInputHandler::HandleKeyEvent(TInt aType, const TKeyEvent& aKeyEvent)
{
	if (aType == EEventKeyDown)
	{
		if (aKeyEvent.iScanCode == DEVICE_UP_SCANCODE)
		{
			iJogdialDownInFrame[0] = iFrames;
		}
		else if (aKeyEvent.iScanCode == DEVICE_DOWN_SCANCODE)
		{
			iJogdialDownInFrame[1] = iFrames;
		}
		PostKeyEvent(MapToKeyCode(aKeyEvent.iScanCode), EFalse);
	}
	else if (aType == EEventKeyUp)
	{
		if ((aKeyEvent.iScanCode != DEVICE_UP_SCANCODE) && (aKeyEvent.iScanCode != DEVICE_DOWN_SCANCODE))
			PostKeyEvent(MapToKeyCode(aKeyEvent.iScanCode), ETrue);
	}
	else if (aType == EEventKey)
	{
		if (isMovingForward && ( (aKeyEvent.iCode==1) || (aKeyEvent.iCode==2) ))
		{
			event_t ev;
			ev.type = ev_joystick;
			ev.data1 = 0;
			ev.data3 = 0;
			ev.data2 = (aKeyEvent.iCode==1) ? 75 : -75;
	//		RDebug::Print(_L("ev_mouse: aX=%d, aY=%d, mouseX=%d, mouseY=%d"), aX, aY, ev.data2, ev.data3);
			D_PostEvent(&ev);
		}
		if (aKeyEvent.iCode == 'p')
		{
			ToggleProfiler();
		}
	}
}

void TDoomControlInputHandler::PostKeyEvent(TInt aKeyCode, TBool aUpEvent)
{
	if (aKeyCode)
	{
		event_t ev;
		ev.type = aUpEvent ? ev_keyup : ev_keydown;
		ev.data1 = aKeyCode;
		ev.data2 = 0;
		ev.data3 = 0;
//		RDebug::Print(_L("D_PostEvent --> key=%i, up=%i"), aKeyCode, (TInt) aUpEvent);
		D_PostEvent(&ev);
	}
}


void TDoomControlInputHandler::PreDrawFrameToScreen(CGameScreenDevice* /*aDstGameScreenDevice*/)
{
	iFrames++;
	if (iJogdialDownInFrame[0] + KDeferJogdialUpEventsDeltaFrames == iFrames)
		PostKeyEvent(MapToKeyCode(DEVICE_UP_SCANCODE), ETrue);
	if (iJogdialDownInFrame[1] + KDeferJogdialUpEventsDeltaFrames == iFrames)
		PostKeyEvent(MapToKeyCode(DEVICE_DOWN_SCANCODE), ETrue);
}


class CSmartPenToMouseInputHandler : public CSmartPenInputHandler
{
public:
	static CSmartPenToMouseInputHandler* NewL(CGameWindow& aGameWindow, CGameController& aGameController);

protected:
	virtual void HandleMoveEvent(TInt aX, TInt aY);
	virtual void HandleUpEvent(){};
	virtual void HandleDownEvent(){};

protected:
	CSmartPenToMouseInputHandler(CGameWindow& aGameWindow, CGameController& aGameController);
	void ConstructL();
};


CSmartPenToMouseInputHandler::CSmartPenToMouseInputHandler(CGameWindow& aGameWindow, CGameController& aGameController)
	: CSmartPenInputHandler(aGameWindow, aGameController)
{
//	iAutoX = EFalse;
}

void CSmartPenToMouseInputHandler::ConstructL()
{
	CSmartPenInputHandler::ConstructL();
}


CSmartPenToMouseInputHandler* CSmartPenToMouseInputHandler::NewL(CGameWindow& aGameWindow, CGameController& aGameController)
{
	CSmartPenToMouseInputHandler*		self = new(ELeave)CSmartPenToMouseInputHandler(aGameWindow, aGameController);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}


void CSmartPenToMouseInputHandler::HandleMoveEvent(TInt aX, TInt aY)
{
	static lastY=0;
	static count = 0;
	if (++count != 3)
		return;
	count = 0;
	TPoint	delta = iLastPenPosition - iDownPenPosition;
	delta = TransformDeltaForOrientation(delta);
	if (abs(delta.iX) < 10)
		delta.iX = 0;
	if (abs(delta.iY) < 10)
		delta.iY = 0;

	aX = delta.iX;
	aY = delta.iY;

	if (aY != 0)
	{
		iDownPenPosition = iLastPenPosition;
		lastY = aY;
	}
	else
	{
		aY = lastY;
	}

	event_t ev;
	ev.type = ev_mouse;
	ev.data1 = 0;
	ev.data3 = (aY) ? ((aY > 0) ?  -250 : 250) : 0;
	if (iEventCount < 3)
		ev.data2 = (aX) ? ((aX > 0) ?  75 : -75) : 0;
	else
		ev.data2 = (aX) ? ((aX > 0) ?  150 : -150) : 0;
//	RDebug::Print(_L("ev_mouse: aX=%d, aY=%d, mouseX=%d, mouseY=%d"), aX, aY, ev.data2, ev.data3);
	D_PostEvent(&ev);
}




#define ROTATION_FILE_NAME "c:\\system\\apps\\edoom\\rotright.ini"
void SaveRotationSetting(TBool aIsLeft)
{
	if (aIsLeft)
	{
		unlink(ROTATION_FILE_NAME);
	}
	else
	{
		FILE* file=fopen(ROTATION_FILE_NAME, "w+");
		if (file)
		{
			fprintf(file, "\n");
			fclose(file);
		}
	}
}

/****************************************************
 *
 * IMPLEMENTATION OF VIDEO ROUTINES
 *
 ****************************************************/

static CGameWindow*				sGameWindow = NULL;
CMemGameScreenDevice*			sMemGameScreenDevice = NULL;
CGamePalette*					sCGamePalette = NULL;
TDoomFrameDrawer				sDoomFrameDrawer;
TDoomControlInputHandler		sDoomControlInputHandler;
CSmartPenToMouseInputHandler*	sSmartPenToMouseInputHandler;
CSmartPenToKeyInputHandler*		sSmartPenToKeyInputHandler;


#define MAXWINMODES 2
static int windowedModes[MAXWINMODES][2] = 
{
	{320, 200},
	{320, 200}
};

static char* windowedModesText[MAXWINMODES] = 
{
	"320x200 L",
	"320x200 R"
};



int VID_NumModes(void)
{
    return MAXWINMODES;
}

void I_StartupGraphics(void)
{
	if( graphics_started )
		return;

		// honor left and right rotation
	if (access(ROTATION_FILE_NAME, 0) == 0)
		VID_SetMode(1);
	else
		VID_SetMode(0);


	graphics_started = true;
}


void I_ReadScreen (byte* scr)
{
	memcpy(scr, vid.buffer, vid.width*vid.height*vid.bpp);
}

void I_ShutdownGraphics(void)
{
    printf( "I_ShutdownGraphics\n");

    if( !graphics_started )
        return;

    graphics_started = false;
}

char *VID_GetModeName (int modenum)
{
   return windowedModesText[modenum];
}


int VID_GetModeForSize( int w, int h)
{
    int i;

    for (i=0; i<MAXWINMODES;i++)
        if(windowedModes[i][0]==w && windowedModes[i][1]==h)
            return i;
    return 0;
}


void I_FinishUpdate(void)
{
	static TInt sCount = 0;
	PROFILER_MARK(PROFILER_6);
	if (sCount++ == 3)
	{
		sCount = 0;
		return;
	}
	sGameWindow->GameFrameTick(sMemGameScreenDevice);
	PROFILER_MARK(PROFILER_END);
}



static int oldMode = -1;

int VID_SetMode(int modenum)
{
	if (modenum >= MAXWINMODES) 
	{
		printf("VID_SetMode modenum >= MAXWINMODES\n");
		return -1;
	}
	if (oldMode == modenum)
		return 1;

	
		/*
		 * First time
		 */
	TBool	isRotLeft = (modenum == 0);
	if (oldMode == -1)
	{
		TSize	gameSize(windowedModes[modenum][0], windowedModes[modenum][1]);
		sGameWindow = CGameWindow::NewL(_L("EDOOM"));
		sGameWindow->SetGameScreenDrawer(&sDoomFrameDrawer);
		sGameWindow->AddScreenOrientationObserverL(sDoomFrameDrawer);
		sGameWindow->GameController().AddGameControlInputHandlerL(sDoomControlInputHandler);
		sGameWindow->AddGameScreenDrawObserverL(sDoomControlInputHandler);
		sGameWindow->AddScreenOrientationObserverL(sDoomControlInputHandler);

		CVirtualKeyboard&		virtualKeyboard = sGameWindow->VirtualKeyboard();
		virtualKeyboard.SetKeyCodesAreScanCodes(ETrue);
		virtualKeyboard.SetPosition(TPoint(0, 50));

		int vidbuffersize = (gameSize.iWidth * gameSize.iHeight * NUMSCREENS)
			+ (gameSize.iWidth * ST_HEIGHT);  //status bar

		vid.width  = gameSize.iWidth;
		vid.height = gameSize.iHeight;
//		printf("VID_SetMode %dx%d\n", vid.width, vid.height);

		vid.direct = vid.buffer = (unsigned char*) malloc(vidbuffersize);
		vid.rowbytes = vid.width;
		vid.bpp      = 1;
		vid.recalc   = 1;

		sMemGameScreenDevice = CMemGameScreenDevice::NewL(TSize(vid.width, vid.height), EColor256, (TUint8*) vid.direct);

//		sSmartPenToMouseInputHandler = CSmartPenToMouseInputHandler::NewL(*sGameWindow, sGameWindow->GameController());
		sSmartPenToKeyInputHandler = CSmartPenToKeyInputHandler::NewL(*sGameWindow, sGameWindow->GameController());
		sSmartPenToKeyInputHandler->AutoForward(ETrue);
	
		CToolbar*	toolbar = CToolbar::NewL(sGameWindow->WsSession(), sGameWindow->GameScreenDevice());
		sGameWindow->AddGameScreenDrawObserverL(*toolbar);
		sGameWindow->AddScreenOrientationObserverL(*toolbar);

		virtualKeyboard.SetVirtualButtons(toolbar->iVirtualButtons);
		virtualKeyboard.SetHotKeyForVirtualKeyboard(KVirtualKeyboardHotkey);
	}
	else
	{
		sGameWindow->ClearScreen();
		SaveRotationSetting(isRotLeft);
	}

	oldMode = modenum;
	sGameWindow->SetScreenOrientation(isRotLeft ? EOrientationRotLeft : EOrientationRotRight);
	vid.modenum  = modenum;

	printf("VID_SetMode(%d) DONE\n", modenum);
	return 1;
}


void I_SetPalette (byte* palette)
{
	ASSERT(sMemGameScreenDevice);


	TDisplayMode	displayMode = sGameWindow->GameScreenDevice()->DisplayMode();
	if (!sCGamePalette)
		sCGamePalette = CGamePalette::NewL(displayMode);

	byte*  usegamma;
	usegamma = gammatable[scr_gamma%5];

	for(int i=0 ; i<256 ; i++)
	{
		int r = usegamma[*palette++];
		int g = usegamma[*palette++];
		int b = usegamma[*palette++];

		TRgb	color(r, g, b);
		(sCGamePalette->PaletteBuffer())[i] = (TUint16) TGameBitmapUtil::GetColorValue(color, displayMode);
	}
	sMemGameScreenDevice->SetPalette(sCGamePalette);


#ifdef __USE_PROFILER
	(sCGamePalette->PaletteBuffer())[255] = (TUint16) TGameBitmapUtil::GetColorValue(KRgbWhite, displayMode);
	video_setup((unsigned*) vid.direct, vid.width, vid.height, 8, 255, 0);
	ToggleProfiler();
#endif
}

int osd_cycles()
{
	return User::TickCount();
}



/*
 * needed in g_game.c 
 */
int IsPenDown()
{
	if (sSmartPenToKeyInputHandler)
		return sSmartPenToKeyInputHandler->IsPenDown();
	return 0;
}
